defmodule NullaWeb.NoteController do
  use NullaWeb, :controller
  alias Nulla.Repo
  alias Nulla.ActivityPub
  alias Nulla.Models.Note
  alias Nulla.Models.InstanceSettings

  def index(conn, _params) do
    notes = Notes.list_notes()
    render(conn, :index, notes: notes)
  end

  def new(conn, _params) do
    changeset = Notes.change_note(%Note{})
    render(conn, :new, changeset: changeset)
  end

  def create(conn, %{"note" => note_params}) do
    case Notes.create_note(note_params) do
      {:ok, note} ->
        conn
        |> put_flash(:info, "Note created successfully.")
        |> redirect(to: ~p"/notes/#{note}")

      {:error, %Ecto.Changeset{} = changeset} ->
        render(conn, :new, changeset: changeset)
    end
  end

  def show(conn, %{"username" => username, "note_id" => note_id}) do
    accept = List.first(get_req_header(conn, "accept"))
    instance_settings = InstanceSettings.get_instance_settings!()
    domain = instance_settings.domain
    note = Note.get_note!(note_id) |> Repo.preload([:user, :media_attachments])

    if username != note.user.username do
      conn
      |> put_status(:not_found)
      |> json(%{error: "Note not found"})
      |> halt()
    end

    if accept in ["application/activity+json", "application/ld+json"] do
      conn
      |> put_resp_content_type("application/activity+json")
      |> json(ActivityPub.note(domain, note))
    else
      render(conn, :show, domain: domain, note: note, layout: false)
    end
  end

  # def show(conn, %{"id" => id}) do
  #  note = Notes.get_note!(id)
  #  render(conn, :show, note: note)
  # end

  def edit(conn, %{"id" => id}) do
    note = Notes.get_note!(id)
    changeset = Notes.change_note(note)
    render(conn, :edit, note: note, changeset: changeset)
  end

  def update(conn, %{"id" => id, "note" => note_params}) do
    note = Notes.get_note!(id)

    case Notes.update_note(note, note_params) do
      {:ok, note} ->
        conn
        |> put_flash(:info, "Note updated successfully.")
        |> redirect(to: ~p"/notes/#{note}")

      {:error, %Ecto.Changeset{} = changeset} ->
        render(conn, :edit, note: note, changeset: changeset)
    end
  end

  def delete(conn, %{"id" => id}) do
    note = Notes.get_note!(id)
    {:ok, _note} = Notes.delete_note(note)

    conn
    |> put_flash(:info, "Note deleted successfully.")
    |> redirect(to: ~p"/notes")
  end
end
