defmodule NullaWeb.UserController do
  use NullaWeb, :controller
  alias Nulla.ActivityPub
  alias Nulla.Utils
  alias Nulla.Models.User
  alias Nulla.Models.Note
  alias Nulla.Models.InstanceSettings

  def show(conn, %{"username" => username}) do
    accept = List.first(get_req_header(conn, "accept"))
    instance_settings = InstanceSettings.get_instance_settings!()
    domain = instance_settings.domain
    user = User.get_user_by_username!(username)
    notes = Note.get_all_notes!(user.id)

    if accept in ["application/activity+json", "application/ld+json"] do
      conn
      |> put_resp_content_type("application/activity+json")
      |> send_resp(200, Jason.encode!(ActivityPub.user(domain, user)))
    else
      following = Utils.count_following_by_username!(user.username)
      followers = Utils.count_followers_by_username!(user.username)

      render(
        conn,
        :show,
        domain: domain,
        user: user,
        notes: notes,
        following: following,
        followers: followers,
        layout: false
      )
    end
  end
end
